facts={
    "soiltype":None,
    'temp':None,
    'water':None,
    'season':None,
}
rules=[
    ({"soiltype":'black','temp':'low','water':'high','season':'monsoon',},'rice'),
    ({"soiltype": 'red', 'temp': 'high', 'water': 'low', 'season': 'summer', }, 'maize'),
    ({"soiltype": 'sandy', 'temp': 'low', 'water': 'high', 'season': 'summer', }, 'ragi'),
    ({"soiltype": 'black', 'temp': 'medium', 'water': 'high', 'season': 'monsoon', }, 'hkj')
]
def inferenceengine(facts,rules):
    recoendations=[]
    for conditions,crops in rules:
        match=True
        for key,values in conditions.items():
            if facts.get(key)!=values:
                match=False
                break
            if match:
               recoendations.append(crops)
        return recoendations
if __name__ == "__main__":
  print('expert system')
#facts['soiltype']=input('enter soiltype').lower()
#facts['temp']=input('enter temp').lower()
#facts['water']=input('enter water').lower()
#facts['season']=input('enter season').lower()
facts['soiltype'] = input('Enter soiltype: ').lower()
facts['temp'] = input('Enter temp: ').lower()
facts['water'] = input('Enter water: ').lower()
facts['season'] = input('Enter season: ').lower()

result=inferenceengine(facts,rules)
if result:
   print('recomendation crops:',','.join(result))





from queue import PriorityQueue
def best(graph,heurestic,start,goal):
    visited=set()
    pq=PriorityQueue()
    pq.put((heurestic[start],start))
    while not pq.empty():
        h,node=pq.get()
        print(f"visiting:{node}")
        if node==goal:
            print("goal reached")
            return True
        visited.add(node)
        for neighbour in graph[node]:
            if neighbour not in visited:
                pq.put((heurestic[neighbour],neighbour))
    print("goal not reached")
    return False
if __name__ == "__main__":
  graph={
    'A':['B','C'],
    'B':['D','E'],
    'C':['F','G'],
    'D':[],
    'E':[],
    'F':[],
    'G':[]
}
  heurestic={
    'A':1,
    'B':2,
    'C':3,
    'D':4,
    'E':5,
    'F':6,
    'G':0
}
  best(graph,heurestic,'A','G')





from queue import PriorityQueue
def astar(graph,h,start,goal):
    pq = PriorityQueue()
    g={start:0}
    pq.put((h[start],start))
    while not pq.empty():
        f,node = pq.get()
        print(f"Visiting {node} (f:{f},g:{g[node]},h:{h[node]})")
        if node == goal:
            print("goal reached")
            return True
        for n,cost in graph[node]:
            new_g=g[node]+cost
            if n not in g or new_g<g[n]:
               g[n]=new_g
               pq.put((new_g+h[n],n))
    print("not reachable")
    return False
if __name__ == "__main__":
  graph = {
    'A': [('B', 7), ('C', 8)],
    'B': [('D', 9), ('E', 10)],
    'C': [('F', 11), ('G', 12)],
    'D': [], 'E': [], 'F': [], 'G': []
}
  h = {'A':1, 'B':2, 'C':3, 'D':4, 'E':5, 'F':6, 'G':0}

  astar(graph, h, 'A', 'G')





def unify(expr1, expr2):
    # Store substitutions (variable replacements)
    subs = {}

    # Both expressions must have same outer symbol
    if expr1[0] != expr2[0]:
        return None

    args1 = expr1[1]
    args2 = expr2[1]

    # Arguments count must match
    if len(args1) != len(args2):
        return None

    # Compare each argument
    for a, b in zip(args1, args2):
        if a == b:
            continue
        elif a.islower():   # if a is a variable (like 'x')
            subs[a] = b
        elif b.islower():   # if b is a variable
            subs[b] = a
        else:
            return None     # can't unify constants or different functions

    return subs
expr1=('knows',['?X','jhon'])
expr2=('knows',['mary','?Y'])
print("Unification:", unify(expr1,expr2 ))



# The functions need to be defined outside the if __name__ block 
# so they are available when called.

# Note: The functions are still implicitly relying on the global variables 
# N and board being defined later, which works in this structure.

def is_attack(i, j):
    for k in range(N):
        if board[i][k] == 1 or board[k][j] == 1:
            return True

        for l in range(N):
            if (k + l == i + j) or (k - l == j - i):
                if board[k][l] == 1:
                    return True
    return False


def solve_queens(n):
    if n == 0:
        return True
    for i in range(N):
        for j in range(N):
            if board[i][j] == 0 and not is_attack(i, j):
                board[i][j] = 1
                if solve_queens(n - 1):
                    return True
                board[i][j] = 0
    return False


# --- Standard Python Entry Point ---

if __name__ == "__main__":
    # The setup logic (which defines the global variables N and board) 
    # is placed inside this block.

    print("enter no of queens:")
    N = int(input())
    board = [[0] * N for _ in range(N)]

    # Now call the main function
    if solve_queens(N):
        for row in board:
            print(row)
    else:
        print("no solution")


graph={
    '5':['7','4'],
    '7':['2','8'],
    '4':[],
    '2':['8'],
    '8':[]
}
visited=set()

def dfs(visited, graph, node):
    if node not in visited :
        print(node,end="  ")
        visited.add(node)
        for node in graph[node]:
            dfs(visited, graph, node)

if __name__ == "__main__":
  print("following is dfs")
dfs(visited,graph,'5')





def cost(H,cond,w=1):
    c={}
    if 'AND' in  cond:
        nodes=cond['AND']
        c['AND' .join(nodes)]=sum(H[n]+w for n in nodes)
    if 'OR' in  cond:
        nodes=cond['OR']
        c['OR' .join(nodes)]=min(H[n]+w for n in nodes)
    return c
def updatecost(H,conds,w=1):
    res={}
    for k in reversed(conds):
        c=cost(H, conds[k], w)
        print(f'{k}: {conds[k]}>>>>>{c}')
        H[k]=min(c.values())

        res[k]=c
    return res
def shortestpath(start,upd,H):
    path=start
    if start in upd:
        vals=list(upd[start].values())
        keys=list(upd[start].keys())
        nxt=keys[vals.index(min(vals))].split()

        if len(nxt) == 1:
            path += '<--' + shortestpath(nxt[0], upd, H)
        else:
            a, b = nxt[0], nxt[-1]
            path += f"<--({keys[0]}) [{shortestpath(a, upd, H)} + {shortestpath(b, upd, H)}]"
    return path

if __name__ == "__main__":
  H = {'A': -1, 'B': 5, 'C': 2, 'D': 4, 'E': 7, 'F': 9, 'G': 3, 'H': 0, 'I': 0, 'J': 0}
  conds = {'A': {'OR': ['B'], 'AND': ['C', 'D']}, 'B': {'OR': ['E', 'F']}, 'C': {'OR': ['G'], 'AND': ['H', 'I']}, 'D': {'OR': ['J']}}
  print('Updated Cost:')
  u = updatecost(H, conds)
  print('*' * 70)
  print('Shortest Path:\n', shortestpath('A', u, H))







def unify(X,Y,subst:None):
    if subst is None:
        subst={}
    if X==Y:
        return subst
    if isinstance(X,str) and X.startswith('?'):
        return unify_var(X,Y,subst)
    if isinstance(Y,str) and Y.startswith('?'):
        return unify_var(Y,X,subst)
    if isinstance(X,list) and isinstance(Y,list) and len(X)==len(Y):
        for a,b in zip(X,Y):
            subst=unify(a,b,subst)
            if subst is None:
                return None
        return subst
    return None
def unify_var(var,val,subst):
    if var in subst:
        return unify(subst[var],val,subst)
    if isinstance(val,str) and val.startswith('?') and val in subst:
        return unify(var,subst[val],subst)
    if occur(var,val,subst):
        return None
    subst[var]=val
    return subst
def occur(var,val,subst):
    if var==val:
        return True
    if isinstance(val,str) and val.startswith('f') and val in subst:
        return occur(var,subst[val],subst)
    if isinstance(val,list):
        return any(occur(var,V,subst) for V in val)
    return False
if __name__ == "__main__":
  expr1=['knows','?X','jhon']
  expr2=['knows','mary','?Y']
print(unify(expr1,expr2,None))



import math
def alphabeta(depth,inx,is_max,values,alpha,beta,max_depth):
    if depth==max_depth:
        return values[inx]
    if is_max:
        for i in range(2):
            vals=alphabeta(depth+1,inx*2+i,False,values,alpha,beta,max_depth)
            alpha=max(alpha,vals)
            if alpha>=beta:
                break
        return alpha
    else:
        for i in range(2):
            vals=alphabeta(depth+1,inx*2+i,True,values,alpha,beta,max_depth)
            beta=min(beta,vals)
            if beta<=alpha:
                break
        return beta
if __name__ == "__main__":

  vals = [3, 5, 6, 9, 1, 2, 0, -1]

  results=alphabeta(0,0,True,vals,-math.inf,math.inf,3)
  print(f'optimal solution:{results}')



# Import the 'main' function from each of your lab files
from .8queen import main as main_8queen
from .alphabeta import main as main_alphabeta
from .ao star import main as main_ao star
from .astar import main as main_astar
from .best import main as main_best
from .bfs import main as main_bfs
from .crop import main as main_crop
from .dfs import main as main_dfs
from .jug import main as main_jug
from .meeting import main as main_meeting
from .unification import main as main_unification

# --- Create Your Simple Functions ---
# Now, create user-friendly function names that "point"
# to those imported main functions.

def 8queen():
    """Runs Lab Program 1"""
    print("--- [Executing Lab 1] ---")
    main_8queen()
    print("--- [Lab 1 Finished] ----")

def alphabeta():
    """Runs Lab Program 2"""
    print("--- [Executing Lab 2] ---")
    main_alphabeta()
    print("--- [Lab 2 Finished] ----")
    
def ao star():
    """Runs Lab Program 3"""
    print("--- [Executing Lab 3] ---")
    main_ao star()
    print("--- [Lab 3 Finished] ----")
def astar():
    """Runs Lab Program 3"""
    print("--- [Executing Lab 3] ---")
    main_astar()
    print("--- [Lab 3 Finished] ----")
def best():
    """Runs Lab Program 3"""
    print("--- [Executing Lab 3] ---")
    main_best()
    print("--- [Lab 3 Finished] ----")
def bfs():
    """Runs Lab Program 3"""
    print("--- [Executing Lab 3] ---")
    main_bfs()
    print("--- [Lab 3 Finished] ----")
def crop():
    """Runs Lab Program 3"""
    print("--- [Executing Lab 3] ---")
    main_crop()
    print("--- [Lab 3 Finished] ----")
def dfs():
    """Runs Lab Program 3"""
    print("--- [Executing Lab 3] ---")
    main_dfs()
    print("--- [Lab 3 Finished] ----")
def jug():
    """Runs Lab Program 3"""
    print("--- [Executing Lab 3] ---")
    main-jug()
    print("--- [Lab 3 Finished] ----")
def meeting():
    """Runs Lab Program 3"""
    print("--- [Executing Lab 3] ---")
    main_meeting()
    print("--- [Lab 3 Finished] ----")
def unification():
    """Runs Lab Program 3"""
    print("--- [Executing Lab 3] ---")
    main_unification()
    print("--- [Lab 3 Finished] ----")
# Add all your other programs here...


jug1,jug2,goal=4,3,2
visited=[[False for _ in range(jug2+1)] for _ in range(jug1+1)]
def waterjug(vol1,vol2):
    if(vol1==goal and vol2==0) or (vol2==goal and vol1==0):
        print(vol1 ,'\t',vol2)
        print('solution found')
        return True
    if visited[vol1][vol2]:
        return False
    visited[vol1][vol2]=True
    print(vol1, '\t', vol2)
    return (
            waterjug(jug1, vol2) or
            waterjug(vol1, jug2) or
        waterjug(0 ,vol2)  or
        waterjug(vol1,0 ) or

        waterjug(vol1 + min(vol2,(jug1 - vol1)),vol2 - min(vol2,(jug1 - vol1))) or
        waterjug(vol1 - min(vol1, (jug2 - vol2)), vol2 +min(vol1, (jug2 - vol2)))
    )
if __name__ == "__main__":
  print('steps')
  print('jug1 \t jug2')
waterjug(0, 0)


import setuptools
import io

# 1. Read the README.md file safely with UTF-8 encoding
try:
    # Use io.open for consistent encoding across different operating systems
    with io.open("README.md", "r", encoding="utf-8") as fh:
        long_description = fh.read()
except FileNotFoundError:
    long_description = ""

# 2. Setup Configuration
setuptools.setup(
    # --- REQUIRED METADATA ---
    name='myailab',  # e.g., 'my-awesome-lib' (used for 'pip install')
    version='0.1.0',  # Start with a low version number
    author='harsha537',
    author_email='harshavardanreddykadapala@gmail.com',
    description='[A short, one-sentence description of your library.]',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://pypi.org/project/myailab/',

    # --- PACKAGE STRUCTURE ---
    # Finds the directory containing __init__.py (e.g., 'my_awesome_lib/')
    packages=setuptools.find_packages(),

    # --- DEPENDENCIES ---
    install_requires=[
        # List external libraries your project needs here:
        # e.g., 'requests>=2.25.1',
        # e.g., 'pandas',
    ],

    # --- CLASSIFIERS & LICENSE ---
    license='MIT',  # Must match the license in your LICENSE file

    # Helps users find your library on PyPI
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],

    # --- MINIMUM PYTHON VERSION ---
    python_requires='>=3.8',
)


availability = {
    "Alice":   [("Monday", "10AM", "Room A"), ("Tuesday", "2PM", "Room B")],
    "Bob":     [("Monday", "10AM", "Room A"), ("Wednesday", "11AM", "Room C")],
    "Charlie": [("Monday", "10AM", "Room A"), ("Tuesday", "2PM", "Room B")],
    "David":   [("Monday", "10AM", "Room A"), ("Thursday", "4PM", "Room D")],
    "Eve":     [("Tuesday", "2PM", "Room B")]

}
def schedule(avail):
    slotcount={}
    for person,slots in avail.items():
        for slot in slots:
            slotcount[slot]=slotcount.get(slot,0)+1
        bestslot=max(slotcount,key=slotcount.get)
        participants=[P for P ,slots in avail.items() if bestslot in slots]
        return bestslot,participants
if __name__ == "__main__":
  slot,participants=schedule(availability)
print('meeting schedule succesfully')
print(f'day:{slot[0]}')
print(f'time:{slot[1]}')
print(f'place:{slot[2]}')
print(f'participants:{participants}')



